package commgrids.nuggets.wiley.rss.settings;

import java.io.FileReader;
import java.io.IOException;
import java.io.StringReader;

import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.StringTokenizer;
import java.util.Vector;

import commgrids.nuggets.wiley.Database;

import commgrids.nuggets.wiley.userconfig.Col;
import commgrids.nuggets.wiley.userconfig.Userconfig;

/*
   import org.w3c.dom.Document;
   import org.w3c.dom.DocumentType;
   import org.w3c.dom.NamedNodeMap;
   import org.w3c.dom.Node;
   import org.w3c.dom.NodeList;
 */

// Import your vendor's DOM parser
import org.apache.xerces.parsers.DOMParser;

import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;

import org.jdom.input.SAXBuilder;

import org.xml.sax.SAXException;


/**
 * <b><code>DOMParserDemo</code></b> will take an XML file and display
 *   the document using DOM.
 *
 * @version 1.0
 */
public class RSSParser {
    public String          topic           = null;
    private Hashtable      lengthHashTable = null;
    private Hashtable      orderHashTable  = null;
    private RSSConstructor rssConstructor  = null;
    private RSSElement     rssElement      = null;
    private String         searchText      = null;
    private String         searchType      = null;
    private Vector         rssVector       = null;
    private boolean        debug           = false;

    /**
     *
     */
    public RSSParser() {
        /*----------------BEGIN DEBUG------------------*
           System.out.println("inside the rss parser constructor");
           /*----------------END DEBUG--------------------*/
        rssConstructor = new RSSConstructor();
    }

    /**
     *
     * @param aRssElement - the new value for rssElement
     */
    public void setRssElement(RSSElement aRssElement) {
        rssElement = aRssElement;
    }

    /**
     *
     * @return the RSSElement value of rssElement.
     */
    public RSSElement getRssElement() {
        return rssElement;
    }

    /**
     *
     * @param aRssVector - the new value for rssVector
     */
    public void setRssVector(Vector aRssVector) {
        rssVector = aRssVector;
    }

    /**
     *
     * @return the Vector value of rssVector.
     */
    public Vector getRssVector() {
        return rssVector;
    }

    /**
     * DOCUMENT ME!
     *
     * @param searchText DOCUMENT ME!
     * @param searchType DOCUMENT ME!
     */
    public void setSearchElements(String searchText,
                                  String searchType) {
        this.searchText = searchText;
        this.searchType = searchType;
    }

    /**
     *
     * @param topicName
     * @return int
     */
    public int getSize(String topicName) {
        topic = topicName;

        int size = 0;

        try {
            DOMParser parser     = new DOMParser();
            Document  doc        = new SAXBuilder().build(new StringReader(rssConstructor.getRSS(
                                                                                   topic)));
            Element   newelement = doc.getRootElement();
            Document  newdoc     = new Document(newelement.getChild("channel"));
            Iterator  i          = newdoc.getRootElement()
                                         .getChildren()
                                         .iterator();

            while (i.hasNext()) {
                Element e = (Element) i.next();

                if (e.getName()
                     .equals("item")) {
                    size++;
                }
            }
        } catch (Exception e) {
            if (debug) {
                System.out.println(this.getClass().getName()
                                   + ".getSize ");
            }
            e.printStackTrace();
        }

        return size;
    }

    /**
     * <p>
     * This parses the file, and then prints the document out
     *   using DOM.
     * </p>
     *
     * @param uri <code>String</code> URI of file to parse.
     */
    public Vector constructRSSVector(Document d) {
        Vector v = null;

        try {
            v = new Vector();

            Element  newe   = d.getRootElement();
            Document newdoc = new Document(newe.getChild("channel"));
            List     list   = newdoc.getRootElement()
                                    .getChildren();
            Iterator i      = newdoc.getRootElement()
                                    .getChildren()
                                    .iterator();

            while (i.hasNext()) {
                Element e = (Element) i.next();

                if (e.getName()
                     .equals("Item")) {
                    RSSElement re = new RSSElement();
                    re.setLink(e.getChildText("link"));
                    re.setLinkOrder(((Integer) orderHashTable.get(
                                                    "link")).intValue());
                    re.setLinkLength(((Integer) lengthHashTable.get(
                                                     "link")).intValue());

                    re.setCategory(e.getChildText("category"));
                    re.setCategoryOrder(((Integer) orderHashTable.get("category")).intValue());
                    re.setCategoryLength(((Integer) lengthHashTable.get("category")).intValue());

                    re.setDate(e.getChildText("date"));
                    re.setDateOrder(((Integer) orderHashTable.get("date")).intValue());
                    re.setDateLength(((Integer) lengthHashTable.get("date")).intValue());

                    re.setArticleId(e.getChildText("articleId"));
                    re.setArticleIdOrder(((Integer) orderHashTable.get(
                                                 "articleId")).intValue());
                    re.setArticleIdLength(((Integer) lengthHashTable.get(
                                                  "articleId")).intValue());

                    re.setAuthor(e.getChildText("author"));
                    re.setAuthorOrder(((Integer) orderHashTable.get("author")).intValue());
                    re.setAuthorLength(((Integer) lengthHashTable.get("author")).intValue());

                    re.setArticleTitle(e.getChildText("articleTitle"));
                    re.setArticleTitleOrder(((Integer) orderHashTable.get("articleTitle")).intValue());
                    re.setArticleTitleLength(((Integer) lengthHashTable.get("articleTitle")).intValue());

                    v.add(re);
                }
            }
        } catch (Exception ex) {
            if (debug) {
                System.out.println(this.getClass().getName()
                                   + ".constructRSSVector ");
            }

            ex.printStackTrace();
        }

        return v;
    }

    /**
     * <p>
     * This parses the file, and then prints the document out
     *   using DOM.
     * </p>
     *
     * @param uri <code>String</code> URI of file to parse.
     */
    public Vector constructRSSVectorWithSearchOptions(Document d) {
        Vector v = null;

        try {
            v = new Vector();

            Element  newe   = d.getRootElement();
            Document newdoc = new Document(newe.getChild("channel"));
            List     list   = newdoc.getRootElement()
                                    .getChildren();
            Iterator i      = newdoc.getRootElement()
                                    .getChildren()
                                    .iterator();

            while (i.hasNext()) {
                Element e = (Element) i.next();

                if (e.getName()
                     .equals("Item")) {
                    RSSElement re = new RSSElement();
                    re.setLink(e.getChildText("link"));
                   re.setLinkOrder(((Integer) orderHashTable.get(
                                                   "link")).intValue());
                   re.setLinkLength(((Integer) lengthHashTable.get(
                                                    "link")).intValue());

                   re.setCategory(e.getChildText("category"));
                   re.setCategoryOrder(((Integer) orderHashTable.get("category")).intValue());
                   re.setCategoryLength(((Integer) lengthHashTable.get("category")).intValue());

                   re.setDate(e.getChildText("date"));
                   re.setDateOrder(((Integer) orderHashTable.get("date")).intValue());
                   re.setDateLength(((Integer) lengthHashTable.get("date")).intValue());

                   re.setArticleId(e.getChildText("articleId"));
                   re.setArticleIdOrder(((Integer) orderHashTable.get(
                                                "articleId")).intValue());
                   re.setArticleIdLength(((Integer) lengthHashTable.get(
                                                 "articleId")).intValue());

                   re.setAuthor(e.getChildText("author"));
                   re.setAuthorOrder(((Integer) orderHashTable.get("author")).intValue());
                   re.setAuthorLength(((Integer) lengthHashTable.get("author")).intValue());

                   re.setArticleTitle(e.getChildText("articleTitle"));
                   re.setArticleTitleOrder(((Integer) orderHashTable.get("articleTitle")).intValue());
                   re.setArticleTitleLength(((Integer) lengthHashTable.get("articleTitle")).intValue());

                    if (searchType.equalsIgnoreCase("link")
                            && searchWordsInText(searchText,
                                                 re.getLink())) {
                        v.add(re);
                    } else if (searchType.equalsIgnoreCase("date")
                                   && searchWordsInText(searchText,
                                                        re.getDate())) {
                        v.add(re);
                    } else if (searchType.equalsIgnoreCase("articleId")
                                   && searchWordsInText(searchText,
                                                        re.getArticleId())) {
                        v.add(re);
                    } else if (searchType.equalsIgnoreCase("author")
                                   && searchWordsInText(searchText,
                                                        re.getAuthor())) {
                        v.add(re);
                    } else if (searchType.equalsIgnoreCase("articleTitle")
                                   && searchWordsInText(searchText,
                                                        re.getArticleTitle())) {
                        v.add(re);
                      } else if (searchType.equalsIgnoreCase("category")
                                     && searchWordsInText(searchText,
                                                          re.getCategory())) {
                          v.add(re);
                      }

                }
            }
        } catch (Exception ex) {
            if (debug) {
                System.out.println(this.getClass().getName()
                                   + ".constructRSSVectorWithSearchOptions ");
            }
            ex.printStackTrace();
        }

        return v;
    }

    /**
     * DOCUMENT ME!
     */
    public void debugOFF() {
        debug = false;
    }

    /**
     * DOCUMENT ME!
     */
    public void debugON() {
        debug = true;
    }

    /**
     * DOCUMENT ME!
     *
     * @param args DOCUMENT ME!
     */
    public static void main(String[] args) {
        try {
            RSSParser rp = new RSSParser();
            rp.parseRSS("","Grids");

            Vector v = rp.rssVector;

            for (int i = 0; i < v.size(); i++) {
                RSSElement re = (RSSElement) v.get(i);

            }
        } catch (Exception e) {
        }
    }

    /**
     * DOCUMENT ME!
     *
     * @param topic DOCUMENT ME!
     */
    public Vector parseRSS(String username,String topic) throws JDOMException {
        this.topic = topic;

        DOMParser parser = new DOMParser();

        try {
            RSSDisplayerPageSettings rdps = new RSSDisplayerPageSettings();
            rdps.getUserConfig(username,topic);
            this.lengthHashTable = rdps.getLengthHashTable();
            this.orderHashTable  = rdps.getOrderHashTable();

            if (rssConstructor == null) {
                System.out.println("rssConstructor is null");
            }

            String databaseResult = rssConstructor.getRSS(topic);

            /*----------------BEGIN DEBUG------------------*/
           /* System.out.println("Topic is ------------------------------> "
                               + topic + "\n" + databaseResult);

            /*----------------END DEBUG--------------------*/
            if ((databaseResult != null) && (databaseResult.length() > 0)) {
                Document doc = new SAXBuilder().build(new StringReader(
                                                              databaseResult));

                if ((searchType != null) && (searchType.length() > 0)
                        && (searchText != null) && (searchText.length() > 0)) {
                    rssVector = this.constructRSSVectorWithSearchOptions(doc);
                } else {
                    rssVector = this.constructRSSVector(doc);
                }

                return rssVector;
            }
        } catch (JDOMException j) {
            if (debug) {
        /*        System.out.println(this.getClass().getName()
                                   + ".parseRSS ");
         */
            }
            j.printStackTrace();
        } catch (Exception e) {
            if (debug) {
            /*    System.out.println(this.getClass().getName()
                                   + ".parseRSS ");
            */
            }
            e.printStackTrace();
        }

        return null;
    }

    /**
     * DOCUMENT ME!
     *
     * @param search DOCUMENT ME!
     * @param text DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public boolean searchWordsInText(String search,
                                     String text) {
        search = search.trim()
                       .toLowerCase();
        text   = text.trim()
                     .toLowerCase();

        StringTokenizer searchString = new StringTokenizer(search, " ");

        while (searchString.hasMoreTokens()) {
            if (text.indexOf(searchString.nextToken().toLowerCase()) >= 0) {
                return true;
            }
        }

        return false;
    }
}