package commgrids.nuggets.wiley;

import org.gjt.mm.mysql.Driver;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.PrintWriter;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

import java.util.Hashtable;
import java.util.Properties;
import java.util.Vector;

/**
 * DOCUMENT ME!
 *
 * @author $author$
 * @version $Revision: 1.4 $
 */
public class Database {
  public static String DEFAULT_PROPERTIES = "metadatabrowser.properties";
  private String      topicFolder               = "";
  private String      dirName               = "";
  private String      resultFolderName               = "";

  private Connection connection = null;
  private Properties properties = null;
  private Statement statement = null;
  private String MySQLDriver = "";
  private String defaultUserConfigFileName = "";
  private String name = "";
  private String passWord = null;
  private String password = "";
  private String url = "";
  private String userName = null;
  private boolean debug = true;

  /**
   * Creates a new Database object.
   */
  public Database() {
    try {
      MySQLDriver = Util.getProperties().getProperty("jdbc.driver");
      url = Util.getProperties().getProperty("jdbc.url");
      name = Util.getProperties().getProperty("jdbc.name");
      password = Util.getProperties().getProperty("jdbc.password");
      defaultUserConfigFileName = Util.getProperties().getProperty(
          "default.userconfiguration.file.name");
      topicFolder =Util.getProperties().getProperty("wiley.topic.folder.name");
      resultFolderName =Util.getProperties().getProperty("wiley.result.folder.name");
    }
    catch (Exception e) {
      e.printStackTrace();
    }
  }

  /**
   * DOCUMENT ME!
   *
   * @return DOCUMENT ME!
   */
  public User[] getAdminList() {
    User users[] = null;
    open();

    try {
      statement = connection.createStatement();

      ResultSet rset;
      String query = "SELECT COUNT(*) AS rowcount FROM "
          + "users AS t1, usernewsgroup AS t2 WHERE "
          + "t1.username=t2.username and t2.admin=" + 1;

      if (debug) {
        System.out.println(this.getClass().getName() + ".getAdminList() "
                           + query);
      }

      rset = statement.executeQuery(query);
      rset.next();

      int resultCount = rset.getInt("rowcount");
      int count = 0;
      rset.close();
      statement.close();
      statement = connection.createStatement();
      query = "SELECT t1.username,t1.firstname,t1.lastname,t2.email,t2.adminname,t2.superadminname "
          + "FROM users AS t1, usernewsgroup AS t2 WHERE "
          + "t1.username=t2.username and " + "t2.admin=" + 1
          + " ORDER BY t1.username";

      if (debug) {
        System.out.println(this.getClass().getName() + ".getAdminList() "
                           + query);
      }

      users = new User[resultCount];
      rset = statement.executeQuery(query);

      while (rset.next()) {
        User user = new User();
        user.setUserName(rset.getString("t1.username"));
        user.setFirstName(rset.getString("t1.firstname"));
        user.setLastName(rset.getString("t1.lastname"));
        user.setEmail(rset.getString("t2.email"));
        user.setAdminName(rset.getString("t2.adminname"));
        user.setSuperAdminName(rset.getString("t2.superadminname"));
        users[count] = user;
        count++;

        if (debug) {
          System.out.println(this.getClass().getName() + ".getAdminList() "
                             + users[count - 1]);
        }
      }

      rset.close();
      statement.close();
      close();

      return users;
    }
    catch (Exception ex) {
      System.err.println(this.getClass().getName() + ".getAdminList() "
                         + "ERROR: Unable to fullfill query");
      ex.printStackTrace();
    }

    return users;
  }

  /**
   * DOCUMENT ME!
   *
   * @param topic DOCUMENT ME!
   *
   * @return DOCUMENT ME!
   */
  public User[] getAdminList(String topic) {
    User users[] = null;
    open();

    try {
      statement = connection.createStatement();

      ResultSet rset;
      String query = "SELECT COUNT(*) AS rowcount "
          + "FROM users AS t1, usernewsgroup AS t2 WHERE "
          + "t1.username=t2.username and t2.topic=\"" + topic
          + "\" and t2.admin=" + 1;

      if (debug) {
        System.out.println(this.getClass().getName()
                           + ".getAdminList(String topic) " + query);
      }

      rset = statement.executeQuery(query);
      rset.next();

      int resultCount = rset.getInt("rowcount");
      int count = 0;
      rset.close();
      statement.close();
      statement = connection.createStatement();
      query =
          "SELECT t1.username,t1.firstname,t1.lastname,t2.email,t2.adminname  "
          + "FROM users AS t1, usernewsgroup AS t2 WHERE "
          + "t1.username=t2.username and t2.topic=\"" + topic
          + "\" and t2.admin=" + 1 + " ORDER BY t1.username";

      if (debug) {
        System.out.println(this.getClass().getName()
                           + ".getAdminList(String topic) " + query);
      }

      users = new User[resultCount];
      rset = statement.executeQuery(query);

      while (rset.next()) {
        User user = new User();
        user.setUserName(rset.getString("t1.username"));
        user.setFirstName(rset.getString("t1.firstname"));
        user.setLastName(rset.getString("t1.lastname"));
        user.setEmail(rset.getString("t2.email"));
        user.setAdminName(rset.getString("t2.adminname"));
        users[count] = user;
        count++;

        if (debug) {
          System.out.println(this.getClass().getName()
                             + ".getAdminList(String topic) "
                             + users[count - 1].getUserName());
        }
      }

      rset.close();
      statement.close();
      close();

      return users;
    }
    catch (Exception ex) {
      System.err.println(this.getClass().getName()
                         + ".getAdminList(String topic) "
                         + "ERROR: Unable to fullfill query");
      ex.printStackTrace();
    }

    return users;
  }

  /**
   * DOCUMENT ME!
   *
   * @param user DOCUMENT ME!
   * @param topic DOCUMENT ME!
   *
   * @return DOCUMENT ME!
   */
  public String getAdminName(String user,
                             String topic) {
    String adminName = "";

    try {
      open();
      statement = connection.createStatement();

      String query = "Select adminname FROM usernewsgroup WHERE username=\""
          + user + "\" and topic=\"" + topic + "\"";

      if (debug) {
        System.out.println(this.getClass().getName()
                           + ".getAdminName(String user,String topic) " + query);
      }

      ResultSet rset = statement.executeQuery(query);
      adminName = (rset.next()
                   ? rset.getString("adminname")
                   : "");
      rset.close();
      statement.close();
      close();
    }
    catch (Exception ex) {
      ex.printStackTrace();
    }

    return adminName;
  }

  /**
   * DOCUMENT ME!
   *
   * @param userName DOCUMENT ME!
   *
   * @return DOCUMENT ME!
   */
  public Topic[] getAdminTopicsList(String userName) {
    Topic topics[] = null;
    open();

    try {
      statement = connection.createStatement();

      ResultSet rset;
      String query = "SELECT COUNT(*) AS rowcount FROM usernewsgroup"
          + " WHERE username=\"" + userName + "\""
          + " and admin=" + 1;

      if (debug) {
        System.out.println(this.getClass().getName()
                           + ".getAdminTopicsList(String userName) " + query);
      }

      rset = statement.executeQuery(query);
      rset.next();

      int resultCount = rset.getInt("rowcount");
      int count = 0;
      rset.close();
      statement.close();
      statement = connection.createStatement();
      query = "SELECT t1.topicid,t1.topicname,t1.topicdescription,"
          + "t2.superadminname "
          + "FROM topics AS t1, usernewsgroup AS t2 WHERE "
          + "t1.topicid=t2.topic and t2.username=\"" + userName
          + "\" and t2.admin=" + 1 + " ORDER BY t1.topicid";

      if (debug) {
        System.out.println(this.getClass().getName()
                           + ".getAdminTopicsList(String userName) " + query
                           + " " + resultCount);
      }

      topics = new Topic[resultCount];
      rset = statement.executeQuery(query);

      while (rset.next()) {
        topics[count] = new Topic(rset.getString("topicid"),
                                  rset.getString("topicname"),
                                  rset.getString("topicdescription"),
                                  rset.getString("superadminname"));
        count++;

        if (debug) {
          System.out.println(this.getClass().getName()
                             + ".getAdminTopicsList(String userName) "
                             + topics[count - 1].getTopicID());
        }
      }

      rset.close();
      statement.close();
      close();

      return topics;
    }
    catch (Exception ex) {
      System.err.println(this.getClass().getName()
                         + ".getAdminTopicsList(String userName) "
                         + "ERROR: Unable to fullfill query");
      ex.printStackTrace();
    }

    return topics;
  }

  /**
   * DOCUMENT ME!
   *
   * @return DOCUMENT ME!
   */
  public String getDefaultUserConfig() {
    try {
      BufferedReader in = new BufferedReader(new FileReader(
          defaultUserConfigFileName));
      String str;
      String output = "";

      while ( (str = in.readLine()) != null) {
        output += (str + "\n");
      }

      in.close();

      return output;
    }
    catch (IOException e) {
      e.printStackTrace();
    }

    return "";
  }

  /**
   * DOCUMENT ME!
   *
   * @param topic DOCUMENT ME!
   *
   * @return DOCUMENT ME!
   */

  public String getMessages() {
    //added for having wiley data for whole year  11/07/2004
    // by Ahmet Topcu atopcu@indiana.edu
    String tempTopic = "";
    boolean isWholeYear = false;
    boolean isAllData = false;
   /* int loc = topic.indexOf("/");

    if ( (topic.length() > 0) && topic.equals("all")) {
      isAllData = true;
    }
    else
    if (loc <= 0) { //check whether is like 2001(13)
      isWholeYear = true;
    }

    */

    String message = null;
    open();

    try {


      /*  if (isWholeYear) {
       query = "SELECT xmldata FROM xmlcontent WHERE topic LIKE '" + topic + "%'";
        } else if (isAllData) {
          query = "SELECT xmldata FROM xmlcontent";
        } else {
       query = "SELECT xmldata FROM xmlcontent WHERE topic=\"" + topic + "\"";
        }
       */
       statement = connection.createStatement();

       ResultSet rset;
       String query = "";

       int k = 1;

      /*      query = "SELECT xmldata FROM xmlcontent WHERE topic='" + topic + "/" +
              params[i] + "'" + " ORDER BY uri";
     */

      query = "SELECT xmldata FROM xmlcontent WHERE topic='" + topicFolder + "'" + " ORDER BY uri";

          if (debug) {
            System.out.println(this.getClass().getName()
                               + ".getMessages(String topic) " + query);
          }

          rset = statement.executeQuery(query);
          message = "";

          while (rset.next()) {
            message = rset.getString("xmldata");

            File file = new File(resultFolderName+"/result" + k + ".txt");
            FileOutputStream foutStream;
            foutStream = new FileOutputStream(file);
            foutStream.write(message.getBytes());
            k++;

          } //end while
          rset.close();


    statement.close();
    return message;
  }

  catch (Exception ex) {
    System.err.println(this.getClass().getName()
                       + ".getMessagesRSSInfo(String topic) "
                       + "ERROR: Unable to fullfill query");
    ex.printStackTrace();
  }
         // rset.close();
         // statement.close();
        close();
  return message;
}

public String getMessagesRSSInfo(String topic) {
  //added for having wiley data for whole year  07/01/2003 by Ahmet Topcu atopcu@indiana.edu
  String tempTopic = "";
  boolean isWholeYear = false;
  boolean isAllData = false;
  int loc = topic.indexOf("/");

  if ( (topic.length() > 0) && topic.equals("all")) {
    isAllData = true;
  }
  else
  if (loc <= 0) { //check whether is like 2001(13)
    isWholeYear = true;
  }

  String rssFile = null;
  open();

  try {
    statement = connection.createStatement();

    ResultSet rset;
    String query = "";

    if (isWholeYear) {
      query = "SELECT rss FROM xmlcontent WHERE topic LIKE '" + topic + "%'";
    }
    else if (isAllData) {
      query = "SELECT rss FROM xmlcontent";
    }
    else {
      query = "SELECT rss FROM xmlcontent WHERE topic=\"" + topic + "\"";
    }

    if (debug) {
      System.out.println(this.getClass().getName()
                         + ".getMessagesRSSInfo(String topic) " + query);
    }

    rset = statement.executeQuery(query);
    rssFile = "";

    while (rset.next()) {
      rssFile += rset.getString("rss");

      /*if (debug) {
          System.out.println(this.getClass().getName() + ".getMessagesRSSInfo(String topic) "
                             + rssFile);
               }*/
    }

    rset.close();
    statement.close();
    close();

    return rssFile;
  }
  catch (Exception ex) {
    System.err.println(this.getClass().getName()
                       + ".getMessagesRSSInfo(String topic) "
                       + "ERROR: Unable to fullfill query");
    ex.printStackTrace();
  }

  return rssFile;
}

/**
 * DOCUMENT ME!
 *
 * @param user DOCUMENT ME!
 *
 * @return DOCUMENT ME!
 */
public boolean isNewAdmin(User user) {
  boolean result = false;

  try {
    statement = connection.createStatement();

    String query = "select * from usernewsgroup where username=\""
        + user.getUserName() + "\" and topic=\"" + user.getTopic()
        + "\"  and admin=1";

    if (debug) {
      System.out.println(this.getClass().getName()
                         + ".isNewAdmin(User user) " + query);
    }

    ResultSet rset = statement.executeQuery(query);
    result = ( ( (rset == null) || !rset.next())
              ? true
              : false);
    rset.close();
    statement.close();

    return result;
  }
  catch (Exception ex) {
    if (debug) {
      System.out.println(this.getClass().getName() + ".isNewAdmin(User user)");
    }

    ex.printStackTrace();
  }

  return result;
}

/**
 * DOCUMENT ME!
 *
 * @param user DOCUMENT ME!
 *
 * @return DOCUMENT ME!
 */
public boolean isNewUser(User user) {
  boolean result = false;

  try {
    statement = connection.createStatement();

    String query = "select * from usernewsgroup where username=\""
        + user.getUserName() + "\" and topic=\"" + user.getTopic()
        + "\"";

    if (debug) {
      System.out.println(this.getClass().getName() + ".isNewUser " + query);
    }

    ResultSet rset = statement.executeQuery(query);
    result = ( ( (rset == null) || !rset.next())
              ? true
              : false);
    rset.close();

    return result;
  }
  catch (Exception ex) {
    if (debug) {
      System.out.println(this.getClass().getName() + ".isNewUser ");
    }

    ex.printStackTrace();
  }

  return result;
}

/**
 * DOCUMENT ME!
 *
 * @param htable DOCUMENT ME!
 *
 * @return DOCUMENT ME!
 */
/**
 * DOCUMENT ME!
 *
 * @param htable DOCUMENT ME!
 *
 * @return DOCUMENT ME!
 */
public Vector getRecordsFromAdminNewsGroup(Hashtable htable) {
  Vector vector = new Vector();

  try {
    open();
    statement = connection.createStatement();

    ResultSet rset;
    String query = "SELECT topicid FROM topics";

    if (debug) {
      System.out.println(this.getClass().getName()
                         + ".getRecordsFromAdminNewsGroup " + query);
    }

    rset = statement.executeQuery(query);

    while (rset.next()) {
      if (!htable.containsKey(rset.getString("topicid"))) {
        vector.add(rset.getString("topicid"));
      }
    }

    //end while
    rset.close();
    statement.close();
    close();
  } //end try
  catch (Exception ex) {
    System.err.println("Unable to fullfill query");
    ex.printStackTrace();
  }

  return vector;
}

/**
 * DOCUMENT ME!
 *
 * @param uname DOCUMENT ME!
 * @param htable DOCUMENT ME!
 *
 * @return DOCUMENT ME!
 */
public Vector getRecordsFromRequestTemp(String uname,
                                        Hashtable htable) {
  Vector vector = null;
  vector = new Vector();

  try {
    open();
    statement = connection.createStatement();

    ResultSet rset;
    String query = "SELECT * FROM requests WHERE username=\"" + uname
        + "\" order by topic";

    if (debug) {
      System.out.println(this.getClass().getName()
                         + ".getRecordsFromRequestTemp " + query);
    }

    rset = statement.executeQuery(query);

    while (rset.next()) {
      newsrequest.User user = new newsrequest.User();
      user.setUsername(rset.getString("username"));
      user.setTopicName(rset.getString("topic"));

      if (debug) {
        System.out.println(this.getClass().getName()
                           + ".getRecordsFromRequestTemp "
                           + rset.getString("topic"));
      }

      user.setEmail(rset.getString("email"));
      user.setRead(rset.getInt("readaccess"));
      user.setWrite(rset.getInt("writeaccess"));
      user.setEmailNotify(rset.getInt("emailnotify"));
      user.setAttachNotify(rset.getInt("attachnotify"));
      user.setCCNotify(rset.getInt("ccnotify"));
      htable.put(rset.getString("topic"), rset.getString("username"));
      vector.add(user);
    }

    //end while
    rset.close();
    statement.close();
    close();
  } //end try
  catch (Exception ex) {
    System.err.println("Unable to fullfill query");
    ex.printStackTrace();
  }

  return vector;
}

/**
 * DOCUMENT ME!
 *
 * @param uname DOCUMENT ME!
 * @param htable DOCUMENT ME!
 * @param checkHashTable DOCUMENT ME!
 *
 * @return DOCUMENT ME!
 */
public Vector getRecordsFromUserNewsGroup(String uname,
                                          Hashtable htable,
                                          boolean checkHashTable) {
  Vector vector = new Vector();

  try {
    open();
    statement = connection.createStatement();

    ResultSet rset;
    String query = "SELECT * FROM usernewsgroup WHERE USERNAME=\"" + uname
        + "\" order by topic";

    if (debug) {
      System.out.println(this.getClass().getName()
                         + ".getRecordsFromUserNewsGroup " + query);
    }

    rset = statement.executeQuery(query);

    while (rset.next()) {
      newsrequest.User user = new newsrequest.User();
      user.setUsername(rset.getString("username"));
      user.setTopicName(rset.getString("topic"));
      user.setEmail(rset.getString("email"));
      user.setRead(rset.getInt("readaccess"));
      user.setWrite(rset.getInt("writeaccess"));
      user.setEmailNotify(rset.getInt("emailnotify"));
      user.setAttachNotify(rset.getInt("attachnotify"));
      user.setCCNotify(rset.getInt("ccnotify"));

      if (checkHashTable) {
        if (!htable.containsKey(rset.getString("topic"))) {
          vector.add(user);
        }
      }
      else {
        htable.put(rset.getString("topic"), rset.getString("username"));
        vector.add(user);
      }
    }

    //end while
    rset.close();
    statement.close();
    close();
  } //end try
  catch (Exception ex) {
    System.err.println("Unable to fullfill query");
    ex.printStackTrace();
  }

  return vector;
}

/**
 * DOCUMENT ME!
 *
 * @return DOCUMENT ME!
 */
public User[] getSuperAdminList() {
  User users[] = null;
  open();

  try {
    statement = connection.createStatement();

    ResultSet rset;
    String query = "SELECT COUNT(*) AS rowcount FROM users WHERE superadmin=1";

    if (debug) {
      System.out.println(this.getClass().getName() + ".getSuperAdminList "
                         + query);
    }

    rset = statement.executeQuery(query);
    rset.next();

    int resultCount = rset.getInt("rowcount");
    int count = 0;
    rset.close();
    statement.close();
    statement = connection.createStatement();
    query = "SELECT username,firstname,lastname,email "
        + "FROM users WHERE superadmin=" + 1 + " ORDER BY username";

    if (debug) {
      System.out.println(this.getClass().getName() + " " + query);
    }

    users = new User[resultCount];
    rset = statement.executeQuery(query);

    while (rset.next()) {
      User user = new User();
      user.setUserName(rset.getString("username"));
      user.setFirstName(rset.getString("firstname"));
      user.setLastName(rset.getString("lastname"));
      user.setEmail(rset.getString("email"));
      users[count] = user;
      count++;

      if (debug) {
        System.out.println(this.getClass().getName() + " " + users[count - 1]);
      }
    }

    rset.close();
    statement.close();
    close();

    return users;
  }
  catch (Exception ex) {
    System.err.println(this.getClass().getName() + " "
                       + "ERROR: Unable to fullfill query");
    ex.printStackTrace();
  }

  return users;
}

/**
 * DOCUMENT ME!
 *
 * @param topic DOCUMENT ME!
 *
 * @return DOCUMENT ME!
 */
public boolean isTopicInserted(String topic) {
  boolean result = true;
  open();

  try {
    statement = connection.createStatement();

    ResultSet rset;
    String query = "SELECT topicid FROM topics WHERE topicid=\"" + topic
        + "\"";

    if (debug) {
      System.out.println(this.getClass().getName() + " " + query);
    }

    rset = statement.executeQuery(query);
    result = rset.next();
    rset.close();
    statement.close();

    return result;
  }
  catch (Exception ex) {
    System.err.println(this.getClass().getName() + " "
                       + "ERROR: Unable to fullfill query");
    ex.printStackTrace();
  }

  close();

  return result;
}

/**
 * DOCUMENT ME!
 *
 * @return DOCUMENT ME!
 */
public Topic[] getTopicList() {
  Topic topics[] = null;
  open();

  try {
    statement = connection.createStatement();

    ResultSet rset;
    String query = "SELECT COUNT(*) AS rowcount FROM topics";

    if (debug) {
      System.out.println(this.getClass().getName() + ".getTopicList " + query);
    }

    rset = statement.executeQuery(query);
    rset.next();

    int resultCount = rset.getInt("rowcount");
    int count = 0;
    rset.close();
    statement.close();
    statement = connection.createStatement();
    query = "SELECT * FROM TOPICS ORDER BY topicname";

    if (debug) {
      System.out.println(this.getClass().getName() + ".getTopicList " + query);
    }

    topics = new Topic[resultCount];
    rset = statement.executeQuery(query);

    while (rset.next()) {
      topics[count] = new Topic(rset.getString("topicid"),
                                rset.getString("topicname"),
                                rset.getString("topicdescription"));
      count++;

      if (debug) {
        System.out.println(this.getClass().getName() + ".getTopicList "
                           + topics[count - 1].getTopicID());
      }
    }

    rset.close();
    statement.close();
    close();

    return topics;
  }
  catch (Exception ex) {
    System.err.println(this.getClass().getName() + ".getTopicList "
                       + "ERROR: Unable to fullfill query");
    ex.printStackTrace();
  }

  return topics;
}

/**
 * DOCUMENT ME!
 *
 * @param username DOCUMENT ME!
 * @param isAdmin DOCUMENT ME!
 *
 * @return DOCUMENT ME!
 */
public String[] getTopicName(String username,
                             int isAdmin) {
  String topics[] = null;
  open();

  try {
    statement = connection.createStatement();

    ResultSet rset;
    String query =
        "SELECT COUNT(*) AS rowcount FROM usernewsgroup WHERE username=\""
        + username + "\" and admin=" + isAdmin + " ORDER BY topic";

    if (debug) {
      System.out.println(this.getClass().getName()
                         + ".getTopicName(String username,int isAdmin) "
                         + query);
    }

    rset = statement.executeQuery(query);
    rset.next();

    int resultCount = rset.getInt("rowcount");
    int count = 0;
    rset.close();
    statement.close();
    statement = connection.createStatement();
    query = "SELECT topic FROM usernewsgroup WHERE username=\""
        + username + "\" and admin=" + isAdmin + " ORDER BY topic";

    if (debug) {
      System.out.println(this.getClass().getName()
                         + ".getTopicName(String username,int isAdmin) "
                         + query);
    }

    topics = new String[resultCount];
    rset = statement.executeQuery(query);

    while (rset.next()) {
      topics[count] = rset.getString("topic");
      count++;

      if (debug) {
        System.out.println(this.getClass().getName()
                           + ".getTopicName(String username,int isAdmin) "
                           + topics[count - 1]);
      }
    }

    rset.close();
    statement.close();
    close();

    return topics;
  }
  catch (Exception ex) {
    System.err.println(this.getClass().getName()
                       + ".getTopicName(String username,int isAdmin) "
                       + "ERROR: Unable to fullfill query");
    ex.printStackTrace();
  }

  return topics;
}

/**
 * DOCUMENT ME!
 *
 * @return DOCUMENT ME!
 */
public String[] getTopicName() {
  String topics[] = null;
  open();

  try {
    statement = connection.createStatement();

    ResultSet rset;
    String query = "SELECT COUNT(*) AS rowcount FROM topics";

    if (debug) {
      System.out.println(this.getClass().getName() + ".getTopicName() "
                         + query);
    }

    rset = statement.executeQuery(query);
    rset.next();

    int resultCount = rset.getInt("rowcount");
    int count = 0;
    rset.close();
    statement.close();
    statement = connection.createStatement();
    query = "SELECT topicid FROM topics ORDER BY topicid";

    if (debug) {
      System.out.println(this.getClass().getName() + ".getTopicName() "
                         + query);
    }

    topics = new String[resultCount];
    rset = statement.executeQuery(query);

    while (rset.next()) {
      topics[count] = rset.getString("topicid");
      count++;

      if (debug) {
        System.out.println(this.getClass().getName() + ".getTopicName() "
                           + topics[count - 1]);
      }
    }

    rset.close();
    statement.close();
    close();

    return topics;
  }
  catch (Exception ex) {
    System.err.println(this.getClass().getName() + ".getTopicName() "
                       + "ERROR: Unable to fullfill query");
    ex.printStackTrace();
  }

  return topics;
}

/**
 * DOCUMENT ME!
 *
 * @return DOCUMENT ME!
 */
public String[] getUriList() {
  String uriLists[] = null;
  open();

  try {
    statement = connection.createStatement();

    ResultSet rset;
    String query = "SELECT COUNT(*) AS rowcount FROM xmlcontent";

    if (debug) {
      System.out.println(this.getClass().getName() + ".getUriList " + query);
    }

    rset = statement.executeQuery(query);
    rset.next();

    int resultCount = rset.getInt("rowcount");
    int count = 0;
    rset.close();
    statement.close();
    statement = connection.createStatement();
    query = "SELECT * FROM xmlcontent ORDER BY uri";

    if (debug) {
      System.out.println(this.getClass().getName() + ".getUriList " + query);
    }

    uriLists = new String[resultCount];
    rset = statement.executeQuery(query);

    while (rset.next()) {
      uriLists[count] = new String(rset.getString("uri"));
      count++;

      if (debug) {
        System.out.println(this.getClass().getName() + ".getUriList "
                           + uriLists[count - 1].length());
      }
    }

    rset.close();
    statement.close();
    close();

    return uriLists;
  }
  catch (Exception ex) {
    System.err.println(this.getClass().getName() + ".getUriList "
                       + "ERROR: Unable to fullfill query");
    ex.printStackTrace();
  }

  return uriLists;
}

/**
 * DOCUMENT ME!
 *
 * @param userAccessRights DOCUMENT ME!
 */
public void getUserAccessRights(UserAccessRights userAccessRights) {
  open();

  try {
    statement = connection.createStatement();

    ResultSet rset;
    String query = "SELECT * FROM usernewsgroup WHERE username=\""
        + userAccessRights.getUserName() + "\" and topic=\""
        + userAccessRights.getTopic() + "\"";

    if (debug) {
      System.out.println(this.getClass().getName()
                         +
          ".getUserAccessRights(UserAccessRights userAccessRights) "
                         + query);
    }

    rset = statement.executeQuery(query);

    while (rset.next()) {
      userAccessRights.setAttachNotify(rset.getInt("attachnotify"));
      userAccessRights.setCCNotify(rset.getInt("ccnotify"));
      userAccessRights.setEmailNotify(rset.getInt("emailnotify"));
      userAccessRights.setReadAccess(rset.getInt("readaccess"));
      userAccessRights.setWriteAccess(rset.getInt("writeaccess"));
      userAccessRights.setAdmin(rset.getInt("admin"));
      userAccessRights.setEmail(rset.getString("email"));
    }

    rset.close();
    statement.close();
  }
  catch (Exception ex) {
    if (debug) {
      System.out.println(this.getClass().getName()
                         +
          ".getUserAccessRights(UserAccessRights userAccessRights) ");
    }

    ex.printStackTrace();
  }

  close();
}

/**
 * DOCUMENT ME!
 *
 * @param username DOCUMENT ME!
 * @param topic DOCUMENT ME!
 *
 * @return DOCUMENT ME!
 */
public String getUserConfiguration(String username,
                                   String topic) {
  // System.out.println("configuration names----------------------------------"+ username+"   "+topic);
  String configFile = null;

  try {
    open();
    statement = connection.createStatement();

    ResultSet rset;
    String query = "SELECT config FROM configuration WHERE username=\""
        + username + "\" and topicname=\"" + topic + "\"";

    //" WHERE topic=\""
    //                 + topic + "\"";
    if (debug) {
      System.out.println(this.getClass().getName()
                         +
                         ".getUserConfiguration(String username,String topic) "
                         + query);
    }

    rset = statement.executeQuery(query);
    configFile = "";

    while (rset.next()) {
      configFile += rset.getString("config");

      if (debug) {
        System.out.println(this.getClass().getName()
                           +
            ".getUserConfiguration(String username,String topic) "
                           + configFile);
      }
    }

    //System.out.println(configFile);
    if ( (configFile == null) || (configFile.length() == 0)) {}

    rset.close();
    statement.close();
    close();

    return configFile;
  }
  catch (Exception ex) {
    System.err.println(this.getClass().getName()
                       + ".getUserConfiguration(String username,String topic) "
                       + "ERROR: Unable to fullfill query");
    ex.printStackTrace();
  }

  return configFile;
}

/**
 * DOCUMENT ME!
 *
 * @param user DOCUMENT ME!
 */
public void getUserInfo(User user) {
  try {
    open();
    statement = connection.createStatement();

    ResultSet rset;
    String query = "SELECT * FROM users WHERE username=\""
        + user.getUserName() + "\"";

    if (debug) {
      System.out.println(this.getClass().getName()
                         + ".getUserInfo(User user) " + query);
    }

    rset = statement.executeQuery(query);

    while (rset.next()) {
      user.setFirstName(rset.getString("firstname"));
      user.setLastName(rset.getString("lastname"));
      user.setEmail(rset.getString("email"));
      user.setAdmin( ( (rset.getInt("admin") == 0)
                      ? false
                      : true));
      user.setSuperAdmin( ( (rset.getInt("superadmin") == 0)
                           ? false
                           : true));

      if (debug) {
        System.out.println(this.getClass().getName()
                           + ".getUserInfo(User user) " + query);
      }
    }

    rset.close();
    statement.close();
    close();
  }
  catch (Exception ex) {
    System.err.println(this.getClass().getName() + ".getUserInfo(User user) "
                       + "ERROR: Unable to fullfill query");
    ex.printStackTrace();
  }
}

/**
 * DOCUMENT ME!
 *
 * @return DOCUMENT ME!
 */
public User[] getUserList() {
  User users[] = null;
  open();

  try {
    statement = connection.createStatement();

    ResultSet rset;
    String query = "SELECT COUNT(*) AS rowcount FROM users";

    if (debug) {
      System.out.println(this.getClass().getName() + ".getUserList() "
                         + query);
    }

    rset = statement.executeQuery(query);
    rset.next();

    int resultCount = rset.getInt("rowcount");
    int count = 0;
    rset.close();
    statement.close();
    statement = connection.createStatement();
    query =
        "SELECT username,firstname,lastname,email FROM users ORDER BY username";

    if (debug) {
      System.out.println(this.getClass().getName() + ".getUserList() "
                         + query);
    }

    users = new User[resultCount];
    rset = statement.executeQuery(query);

    while (rset.next()) {
      User user = new User();
      user.setUserName(rset.getString("username"));
      user.setFirstName(rset.getString("firstname"));
      user.setLastName(rset.getString("lastname"));
      user.setEmail(rset.getString("email"));
      users[count] = user;
      count++;

      if (debug) {
        System.out.println(this.getClass().getName() + ".getUserList() "
                           + users[count - 1].getUserName());
      }
    }

    rset.close();
    statement.close();
    close();

    return users;
  }
  catch (Exception ex) {
    System.err.println(this.getClass().getName() + ".getUserList() "
                       + "ERROR: Unable to fullfill query");
    ex.printStackTrace();
  }

  return users;
}

/**
 * DOCUMENT ME!
 *
 * @param adminname DOCUMENT ME!
 *
 * @return DOCUMENT ME!
 */
public String[] getUserNames(String adminname) {
  String array[] = null;

  try {
    open();
    statement = connection.createStatement();

    ResultSet rset;
    String query = "SELECT COUNT(DISTINCT t1.username) AS rowcount FROM "
        + "usernewsgroup t1, usernewsgroup t2 WHERE "
        + "t2.username=\"" + adminname
        + "\" AND t1.admin= 0 AND t2.admin=1 AND t1.topic=t2.topic";

    if (debug) {
      System.out.println(this.getClass().getName() + " " + query);
    }

    rset = statement.executeQuery(query);
    rset.next();

    int resultCount = rset.getInt("rowcount");
    array = new String[resultCount];

    int count = 0;
    rset.close();
    query = "SELECT DISTINCT t1.username FROM usernewsgroup t1, "
        + "usernewsgroup t2 WHERE " + "t2.username=\"" + adminname
        + "\" AND t1.admin=0 AND t2.admin=1 AND t1.topic=t2.topic";

    if (debug) {
      System.out.println(this.getClass().getName() + ".getUserNames " + query);
    }

    rset = (ResultSet) statement.executeQuery(query);

    while (rset.next()) {
      if (rset.getString("username").equalsIgnoreCase(adminname)) {
        array[count] = adminname;
      }
      else {
        array[count] = adminname + "/" + rset.getString("username");
      }

      count++;
    }

    //end while
    rset.close();
    statement.close();
    close();
  } //end try
  catch (Exception ex) {
    System.err.println("Unable to fullfill query");
    ex.printStackTrace();
  }

  return array;
}

/**
 * DOCUMENT ME!
 *
 * @param username DOCUMENT ME!
 *
 * @return DOCUMENT ME!
 */
public String[] getUserTopicsList(String username) {
  String topics[] = null;
  open();

  try {
    statement = connection.createStatement();

    ResultSet rset;
    String query =
        "SELECT COUNT(*) AS rowcount FROM usernewsgroup WHERE username=\""
        + username + "\"" + " ORDER BY topic";

    if (debug) {
      System.out.println(this.getClass().getName()
                         + ".getUserTopicsList(String username) " + query);
    }

    rset = statement.executeQuery(query);
    rset.next();

    int resultCount = rset.getInt("rowcount");
    int count = 0;
    rset.close();
    statement.close();
    statement = connection.createStatement();
    query = "SELECT topic FROM usernewsgroup WHERE username=\""
        + username + "\" ORDER BY topic";

    if (debug) {
      System.out.println(this.getClass().getName()
                         + ".getUserTopicsList(String username) " + query);
    }

    topics = new String[resultCount];
    rset = statement.executeQuery(query);

    while (rset.next()) {
      topics[count] = rset.getString("topic");
      count++;

      if (debug) {
        System.out.println(this.getClass().getName()
                           + ".getUserTopicsList(String username) "
                           + topics[count - 1]);
      }
    }

    rset.close();
    statement.close();
    close();

    return topics;
  }
  catch (Exception ex) {
    System.err.println(this.getClass().getName()
                       + ".getUserTopicsList(String username) "
                       + "ERROR: Unable to fullfill query");
    ex.printStackTrace();
  }

  return topics;
}

/**
 * DOCUMENT ME!
 *
 * @param uri DOCUMENT ME!
 *
 * @return DOCUMENT ME!
 */
public boolean isXMLContentInserted(String uri) {
  boolean result = true;
  open();

  try {
    statement = connection.createStatement();

    ResultSet rset;
    String query = "SELECT uri FROM xmlcontent WHERE uri=\"" + uri + "\"";

    if (debug) {
      System.out.println(this.getClass().getName()
                         + ".isXMLContentInserted(String uri)" + query);
    }

    rset = statement.executeQuery(query);
    result = rset.next();
    rset.close();
    statement.close();
    close();

    return result;
  }
  catch (Exception ex) {
    System.err.println(this.getClass().getName()
                       + ".isXMLContentInserted(String uri) "
                       + "ERROR: Unable to fullfill query");
    ex.printStackTrace();
  }

  return result;
}

/**
 * DOCUMENT ME!
 *
 * @param uri DOCUMENT ME!
 *
 * @return DOCUMENT ME!
 */
public String getXMLMetaData(String topic) {
  String metadata = null;
  open();

  try {
    statement = connection.createStatement();

    ResultSet rset;
    String query = "SELECT xmldata FROM xmlcontent";

    // WHERE uri=\"" + uri + "\"";

    if (debug) {
      System.out.println(this.getClass().getName()
                         + ".getXMLMetaData(String uri) " + query);
    }

    rset = statement.executeQuery(query);
    metadata = "";

    while (rset.next()) {
      metadata += rset.getString("xmldata");

      /*if (debug) {
          System.out.println(this.getClass().getName()
                             + ".getXMLMetaData(String uri) "
                             + metadata);
               }*/
    }

    rset.close();
    statement.close();
    close();

    return metadata;
  }
  catch (Exception ex) {
    System.err.println(this.getClass().getName()
                       + ".getXMLMetaData(String uri) "
                       + "ERROR: Unable to fullfill query");
    ex.printStackTrace();
  }

  return metadata;
}
public String getTopicMessage(String topic) {
  String metadata = null;
  open();

  try {
    statement = connection.createStatement();

    ResultSet rset;
    String query = "SELECT xmldata FROM xmlcontent WHERE uri=\""+topic+"\"";

    // WHERE uri=\"" + uri + "\"";

    if (debug) {
      System.out.println(this.getClass().getName()
                         + ".getTopicMessage(String topic) " + query);
    }

    rset = statement.executeQuery(query);
    metadata = "";

    while (rset.next()) {
      metadata = rset.getString("xmldata");

      /*if (debug) {
          System.out.println(this.getClass().getName()
                             + ".getXMLMetaData(String uri) "
                             + metadata);
               }*/
    }

    rset.close();
    statement.close();
    close();

    return metadata;
  }
  catch (Exception ex) {
    System.err.println(this.getClass().getName()
                       + ".getXMLMetaData(String uri) "
                       + "ERROR: Unable to fullfill query");
    ex.printStackTrace();
  }

  return metadata;
}


/**
 * DOCUMENT ME!
 *
 * @param user DOCUMENT ME!
 */
public void addAdmin(User user) {
  try {
    open();

    String query = "";

    if (isNewAdmin(user)) {
      statement = connection.createStatement();
      query = "INSERT INTO usernewsgroup (username,topic,email,"
          + "adminname,readaccess,writeaccess,admin,emailnotify,"
          + "attachnotify,ccnotify,userdisplay,"
          + "admindisplay,superadmindisplay,superadminname) VALUES("
          + "\"" + user.getUserName() + "\"" + "," + "\""
          + user.getTopic() + "\"" + "," + "\"" + user.getEmail()
          + "\"" + "," + "\"" + user.getAdminName() + "\"" + "," + 1
          + "," + 1 + "," + 1 + "," + 1 + "," + 0 + "," + 0 + "," + 1
          + "," + 1 + "," + 1 + ",\"" + user.getAdminName() + "\")";

      if (debug) {
        System.out.println(this.getClass().getName()
                           + ".addAdmin(User user) " + query);
      }

      statement.executeUpdate(query);
      this.insertDefaultUserConfig(user);
    }
    else {
      query = "UPDATE usernewsgroup SET admin=1 WHERE username=\""
          + user.getUserName() + "\" and topic=\"" + user.getTopic()
          + "\"";

      if (debug) {
        System.out.println(this.getClass().getName()
                           + ".addAdmin(User user) " + query);
      }

      statement.executeUpdate(query);
    }

    statement = connection.createStatement();
    query = "UPDATE users SET admin=1 WHERE username=\""
        + user.getUserName() + "\"";
    statement.executeUpdate(query);
    statement.close();
    close();
  }
  catch (Exception ex) {
    ex.printStackTrace();
  }
}

/**
 * DOCUMENT ME!
 *
 * @param topic DOCUMENT ME!
 */
public void addTopic(Topic topic) {
  try {
    open();
    statement = connection.createStatement();

    String query =
        "INSERT INTO topics (topicid,topicname,topicdescription) VALUES("
        + "\"" + topic.getTopicID() + "\",\"" + topic.getTopicName()
        + "\",\"" + topic.getTopicDescription() + "\")";

    if (debug) {
      System.out.println(this.getClass().getName()
                         + ".addTopic(Topic topic) " + query);
    }

    statement.executeUpdate(query);
    statement.close();
    close();
  }
  catch (Exception ex) {
    ex.printStackTrace();
  }
}

/**
 * DOCUMENT ME!
 */
public void close() {
  try {
    this.connection.close();

    if (debug) {
      System.out.println(this.getClass().getName() + ".close() "
                         + "Database connection terminated");
    }
  }
  catch (Exception ex) {
    System.err.println(this.getClass().getName() + ".close() "
                       +
        "ERROR: problem occured while disconnecting from database");
    ex.printStackTrace();
  }
}

/**
 * DOCUMENT ME!
 */
public void debugOFF() {
  debug = false;
}

/**
 * DOCUMENT ME!
 */
public void debugON() {
  debug = true;
}

/**
 * DOCUMENT ME!
 *
 * @param strArray DOCUMENT ME!
 *
 * @return DOCUMENT ME!
 */
public String[] decode(String strArray[]) {
  for (int i = 0; i < strArray.length; i++) {
    strArray[i] = java.net.URLDecoder.decode(strArray[i]);
  }

  return strArray;
}

/**
 * DOCUMENT ME!
 *
 * @param user DOCUMENT ME!
 */
public void deleteAdmin(User user) {
  try {
    open();
    this.deleteUserConfig(user);
    statement = connection.createStatement();

    String query = "DELETE FROM usernewsgroup WHERE username=\""
        + user.getUserName() + "\" and topic=\"" + user.getTopic()
        + "\"";

    if (debug) {
      System.out.println(this.getClass().getName()
                         + ".deleteAdmin(User user) " + query);
    }

    statement.executeUpdate(query);
    statement.close();
    statement = connection.createStatement();

    int length = getTopicName(user.getUserName(), 1).length;
    int isAdmin = 0;

    if (length != 0) {
      isAdmin = 1;
    }

    query = "UPDATE users SET admin=" + isAdmin + " WHERE username=\""
        + user.getUserName() + "\"";

    if (debug) {
      System.out.println(this.getClass().getName()
                         + ".deleteAdmin(User user) " + query);
    }

    statement.executeUpdate(query);
    statement.close();
    close();
  }
  catch (Exception ex) {
    ex.printStackTrace();
  }
}

/**
 * DOCUMENT ME!
 *
 * @param topic DOCUMENT ME!
 */
public void deleteTopic(Topic topic) {
  try {
    open();
    statement = connection.createStatement();

    String query = "DELETE FROM topics WHERE topicID=\"" + topic.getTopicID()
        + "\"";

    if (debug) {
      System.out.println(this.getClass().getName()
                         + ".deleteTopic(Topic topic) " + query);
    }

    statement.executeUpdate(query);
    statement.close();
    close();
  }
  catch (Exception ex) {
    ex.printStackTrace();
  }
}

/**
 * DOCUMENT ME!
 *
 * @param topic DOCUMENT ME!
 * @param userName DOCUMENT ME!
 */
public void deleteTopic(String topic,
                        String userName) {
  User user = new User();
  user.setUserName(userName);
  user.setTopic(topic);
  deleteAdmin(user);
}

/**
 * DOCUMENT ME!
 *
 * @param user DOCUMENT ME!
 */
public void deleteUserConfig(User user) {
  try {
    statement = connection.createStatement();

    String query = "DELETE FROM configuration WHERE username=\""
        + user.getUserName() + "\" and topicname=\""
        + user.getTopic() + "\"";

    if (debug) {
      System.out.println(this.getClass().getName()
                         + ".deleteUserConfig(User user) " + query);
    }

    statement.executeUpdate(query);
    statement.close();
  }
  catch (Exception ex) {
    ex.printStackTrace();
  }
}

/**
 * DOCUMENT ME!
 *
 * @param strArray DOCUMENT ME!
 *
 * @return DOCUMENT ME!
 */
public String[] encode(String strArray[]) {
  for (int i = 0; i < strArray.length; i++) {
    strArray[i] = java.net.URLEncoder.encode(strArray[i]);
  }

  return strArray;
}

/**
 * DOCUMENT ME!
 *
 * @param user DOCUMENT ME!
 */
public void insertDefaultUserConfig(User user) {
  try {
    statement = connection.createStatement();

    String query = "Select config FROM configuration WHERE username=\""
        + user.getUserName() + "\" and topicname=\""
        + user.getTopic() + "\"";

    if (debug) {
      System.out.println(this.getClass().getName()
                         + ".insertDefaultUserConfig(User user) " + query);
    }

    ResultSet rset = statement.executeQuery(query);
    String defaultUserConfig = "";

    if (!rset.next()) {
      defaultUserConfig = this.getDefaultUserConfig();
      defaultUserConfig = this.searchReplace(defaultUserConfig, "\"", "\\\"",
                                             0);
      statement.executeUpdate("INSERT INTO configuration "
                              + "(username,topicname,config) VALUES(\""
                              + user.getUserName() + "\",\""
                              + user.getTopic() + "\",\"" + defaultUserConfig
                              + "\"" + ")");
    }

    rset.close();
  }
  catch (Exception ex) {
    if (debug) {
      System.out.println(this.getClass().getName()
                         + ".insertDefaultUserConfig(User user) ");
    }

    ex.printStackTrace();
  }
}

/**
 * DOCUMENT ME!
 *
 * @param topicname DOCUMENT ME!
 */
/**
 * DOCUMENT ME!
 *
 * @param topicname DOCUMENT ME!
 */
public void insertTopic(String topicname) {
  topicname = searchReplace(topicname, "\"", "\\\"", 0);
  open();

  try {
    statement = connection.createStatement();

    String query = "INSERT INTO topics (topicid) VALUES(\"" + topicname
        + "\")";

    if (debug) {
      System.out.println(this.getClass().getName()
                         + ".insertTopic(String topicname) " + query);
    }

    statement.executeUpdate(query);
    statement.close();
  }
  catch (Exception ex) {
    System.err.println(this.getClass().getName()
                       + ".insertTopic(String topicname) "
                       + "ERROR: Unable to fullfill query");
    ex.printStackTrace();
  }

  close();
}

/**
 * DOCUMENT ME!
 *
 * @param registration DOCUMENT ME!
 *
 * @return DOCUMENT ME!
 */
public boolean insertUser(Registration registration) {
  boolean success = true;
  open();

  try {
    statement = connection.createStatement();

    String roles = "<roles>\n<role>user</role>\n</roles>";
    String query =
        "INSERT INTO users (username,password,firstname,lastname,email) VALUES("
        + "\"" + registration.getUserName() + "\", " + "\""
        + registration.getPassword() + "\", " + "\""
        + registration.getFirstName() + "\", " + "\""
        + registration.getLastName() + "\", " + "\""
        + registration.getEmail() + "\")";

    if (debug) {
      System.out.println(this.getClass().getName()
                         + ".insertUser(Registration registration) " + query);
    }

    statement.executeUpdate(query);
    statement.close();
    close();
  }
  catch (Exception ex) {
    System.err.println(this.getClass().getName()
                       + ".insertUser(Registration registration) "
                       + "ERROR: Unable to fullfill query");
    ex.printStackTrace();
    success = false;
  }

  return success;
}

/**
 * DOCUMENT ME!
 *
 * @param username DOCUMENT ME!
 * @param topicname DOCUMENT ME!
 * @param xmlUserConfig DOCUMENT ME!
 */
public void insertUserConfiguration(String username,
                                    String topicname,
                                    String xmlUserConfig) {
  try {
    open();

    if (false) {
      statement = connection.createStatement();
      xmlUserConfig = this.searchReplace(xmlUserConfig, "\"", "\\\"", 0);

      String query =
          "INSERT INTO configuration (username,topicname,config) VALUES("
          + "\"" + username + "\",\"" + topicname + "\",\""
          + xmlUserConfig + "\")";

      if (debug) {
        System.out.println(this.getClass().getName()
                           + ".insertUserConfiguration(String username,String topicname,String xmlUserConfig) "
                           + query);
      }

      statement.executeUpdate(query);
      statement.close();
      close();
    }
    else {
      this.updateUserConfiguration(username, topicname, xmlUserConfig);
    }
  }
  catch (Exception ex) {
    System.err.println(this.getClass().getName()
                       + ".insertUserConfiguration(String username,String topicname,String xmlUserConfig) "
                       + "ERROR: Unable to fullfill query");
    ex.printStackTrace();
  }
}

/**
 * DOCUMENT ME!
 *
 * @param topicname DOCUMENT ME!
 * @param uri DOCUMENT ME!
 * @param rss DOCUMENT ME!
 * @param xmlcon DOCUMENT ME!
 */
public void insertXMLContent(String topicname,
                             String uri,
                             String rss,
                             String xmlcon) {
  try {
    if ( (topicname != null) && (topicname.length() > 0)) {
      if (!this.isTopicInserted(topicname)) {
        this.insertTopic(topicname);
      }

      if (!this.isXMLContentInserted(uri)) {
        open();
        statement = connection.createStatement();

        //rss       = searchReplace(rss.substring(rss.indexOf("<Item")),
        rss = searchReplace(rss.substring(rss.indexOf("<Item")), "\"",
                            "\\\"", 0);
        xmlcon = searchReplace(xmlcon, "\"", "\\\"", 0);
        // xmlcon =searchReplace(xmlcon, "&", "&amp;",0);

        String query = "INSERT INTO xmlcontent (topic,uri,rss,xmldata) VALUES("
            + "\"" + topicname + "\",\"" + uri + "\",\"" + rss
            + "\",\"" + xmlcon + "\")";

        if (debug) {
          System.out.println(this.getClass().getName() + " " + query);
        }

        statement.executeUpdate(query);
        statement.close();
        close();
      }
      else {
        this.updateXMLContent(topicname, uri, rss, xmlcon);
      }
    }
  }
  catch (Exception ex) {
    System.err.println(this.getClass().getName() + " "
                       + "ERROR: Unable to fullfill query");
    ex.printStackTrace();
  }
}

/**
 * DOCUMENT ME!
 *
 * @param args DOCUMENT ME!
 *
 * @throws SQLException DOCUMENT ME!
 * @throws IOException DOCUMENT ME!
 */
public static void main(String args[]) throws SQLException, IOException {
  Database database = new Database();

  String message = database.getMessages();


}

/**
 * DOCUMENT ME!
 *
 * @param login DOCUMENT ME!
 *
 * @return DOCUMENT ME!
 */
public boolean loginValidate(Login login) {
  open();

  boolean valid = false;

  try {
    statement = connection.createStatement();

    ResultSet rset;
    String query = "SELECT username,password FROM users WHERE username=\""
        + login.getUserName() + "\"";

    if (debug) {
      System.out.println(this.getClass().getName() + " " + query);
    }

    rset = statement.executeQuery(query);

    if (rset.next()) {
      if (rset.getString("password").compareTo(login.getPassword()) == 0) {
        valid = true;
      }
      else {
        login.setPassword("");
        login.setPasswordError("Invalid password");
      }
    }
    else {
      login.setUserName("");
      login.setUserNameError("Invalid username");
    }

    rset.close();
    statement.close();
    close();

    return valid;
  }
  catch (Exception ex) {
    System.err.println(this.getClass().getName() + " "
                       + "ERROR: Unable to fullfill query");
    ex.printStackTrace();
    valid = false;
  }

  return valid;
}

/**
 * DOCUMENT ME!
 */
public void open() {
  try {
    Class.forName(MySQLDriver).newInstance();
    connection = DriverManager.getConnection(url, name, password);

    if (debug) {
      System.out.println(this.getClass().getName() + ".open() "
                         + "Database connection established");
    }
  }
  catch (Exception ex) {
    System.err.println(this.getClass().getName() + ".open() "
                       + "ERROR: problem occured while connecting to database");
    ex.printStackTrace();
    connection = null;
  }
}

/**
 * DOCUMENT ME!
 *
 * @param strArray DOCUMENT ME!
 *
 * @return DOCUMENT ME!
 */
public String[] sort(String strArray[]) {
  encode(strArray);
  java.util.Arrays.sort(strArray);

  return decode(strArray);
}

/**
 * DOCUMENT ME!
 *
 * @param vector DOCUMENT ME!
 */
public void updateRequestTemp(Vector vector) {
  try {
    open();
    statement = connection.createStatement();

    String query = null;

    for (int i = 0; i < vector.size(); i++) {
      newsrequest.User user = (newsrequest.User) vector.get(i);

      /*
         query = "SELECT admin FROM usernewsgroup WHERE username=\""
                 + user.getUsername() + "\" and topic=\""
                 + user.getTopicName() + "\"";
         if (debug) {
             System.out.println(this.getClass().getName()
                                + ".updateRequestTemp " + query);
         }
         ResultSet rset = statement.executeQuery(query);
         if (rset.next()) {
             user.setAdmin(rset.getInt("admin"));
         }
       */
      query = "DELETE FROM requests WHERE username=\"" + user.getUsername()
          + "\" and topic=\"" + user.getTopicName() + "\"";

      if (debug) {
        System.out.println(this.getClass().getName() + ".updateRequestTemp "
                           + query);
      }

      statement.executeUpdate(query);
      System.out.println(user.toString());
      query = "INSERT INTO requests (username,topic,readaccess,"
          + "writeaccess,emailnotify,attachnotify,"
          + "ccnotify,email,admin) VALUES(" + "\"" + user.getUsername()
          + "\",\"" + user.getTopicName() + "\"," + user.getRead() + ","
          + user.getWrite() + "," + user.getEmailNotify() + ","
          + user.getAttachNotify() + "," + user.getCCNotify() + ",\""
          + user.getEmail() + "\"," + user.getAdmin() + ")";

      if (debug) {
        System.out.println(this.getClass().getName() + ".updateRequestTemp "
                           + query);
      }

      statement.executeUpdate(query);

      //System.out.println("end of deletion");
    }

    statement.close();
    close();
  }
  catch (Exception ex) {
    System.err.println("Unable to delete data from the database");
    ex.printStackTrace();
  }
}

/**
 * DOCUMENT ME!
 *
 * @param username DOCUMENT ME!
 * @param topicname DOCUMENT ME!
 * @param xmlUserConfig DOCUMENT ME!
 */
public void updateUserConfiguration(String username,
                                    String topicname,
                                    String xmlUserConfig) {
  try {
    if (!this.isTopicInserted(topicname)) {
      this.insertTopic(topicname);
    }

    open();
    statement = connection.createStatement();
    xmlUserConfig = searchReplace(xmlUserConfig, "\"", "\\\"", 0);

    String query = "UPDATE configuration SET" + " config=\"" + xmlUserConfig
        + "\"" + " WHERE username=\"" + username
        + "\" and topicname=\"" + topicname + "\"";

    if (debug) {
      System.out.println(this.getClass().getName() + " " + query);
    }

    statement.executeUpdate(query);
    statement.close();
    close();
  }
  catch (Exception ex) {
    System.err.println(this.getClass().getName() + " "
                       + "ERROR: Unable to fullfill query");
    ex.printStackTrace();
  }
}

/**
 * DOCUMENT ME!
 *
 * @param topicname DOCUMENT ME!
 * @param uri DOCUMENT ME!
 * @param rss DOCUMENT ME!
 * @param xmlcon DOCUMENT ME!
 */
public void updateXMLContent(String topicname,
                             String uri,
                             String rss,
                             String xmlcon) {
  try {
    if (!this.isTopicInserted(topicname)) {
      this.insertTopic(topicname);
    }

    open();
    statement = connection.createStatement();
    rss = searchReplace(rss.substring(rss.indexOf("<Item")), "\"",
                        "\\\"", 0);
    xmlcon = searchReplace(xmlcon, "\"", "\\\"", 0);

    String query = "UPDATE xmlcontent SET" + " topic=\"" + topicname
        + "\", uri=\"" + uri + "\", rss=\"" + rss
        + "\", xmldata=\"" + xmlcon + "\"" + " WHERE uri=\"" + uri
        + "\"";

    if (debug) {
      System.out.println(this.getClass().getName() + " " + query);
    }

    statement.executeUpdate(query);
    statement.close();
    close();
  }
  catch (Exception ex) {
    System.err.println(this.getClass().getName() + " "
                       + "ERROR: Unable to fullfill query");
    ex.printStackTrace();
  }
}

/**
 * DOCUMENT ME!
 *
 * @param login DOCUMENT ME!
 *
 * @return DOCUMENT ME!
 */
public boolean validateUsername(Login login) {
  open();

  boolean valid = false;

  try {
    statement = connection.createStatement();

    ResultSet rset;
    String query = "SELECT username FROM users WHERE username=\""
        + login.getUserName() + "\"";

    if (debug) {
      System.out.println(this.getClass().getName() + " " + query);
    }

    rset = statement.executeQuery(query);

    if (rset.next()) {
      login.setUserName("");
      login.setUserNameError(
          "This username has already been in use. Please select a new one.");
    }
    else {
      valid = true;
    }

    rset.close();
    statement.close();
    close();

    return valid;
  }
  catch (Exception ex) {
    System.err.println(this.getClass().getName() + " "
                       + "ERROR: Unable to fullfill query");
    ex.printStackTrace();
    valid = false;
  }

  return valid;
}

/**
 * DOCUMENT ME!
 *
 * @param data DOCUMENT ME!
 * @param find DOCUMENT ME!
 * @param replace DOCUMENT ME!
 * @param index DOCUMENT ME!
 *
 * @return DOCUMENT ME!
 */
protected static String searchReplace(String data,
                                      String find,
                                      String replace,
                                      int index) {
  StringBuffer sb = new StringBuffer();
  int a = index;
  int b;
  int findLength = find.length();
  sb.append(data.substring(0, a));

  while ( (b = data.indexOf(find, a)) != -1) {
    sb.append(data.substring(a, b));
    sb.append(replace);
    a = b + findLength;
  }

  if (a < data.length()) {
    sb.append(data.substring(a));
  }

  return sb.toString();
}

// Utility function to read a line from standard input
static String readEntry(String prompt) {
  try {
    StringBuffer buffer = new StringBuffer();
    System.out.print(prompt);
    System.out.flush();

    int c = System.in.read();

    while ( (c != '\n') && (c != -1)) {
      buffer.append( (char) c);
      c = System.in.read();
    }

    return buffer.toString().trim();
  }
  catch (IOException e) {
    return "";
  }
}
}
